zn.define([
    '../service/WFInstanceService.js'
], function (WFInstanceService) {

    return zn.Controller('wfinstance',{
        methods: {
            init: function (){
                this.instanceService = new WFInstanceService(this._store);
            },
            getInstances: {
                method: 'GET/POST',
                value: function (request, response, chain){
                    var _where = [
                        'parent_id = 0'
                    ];
                    if(request.getValue('index_id')){
                        _where.push(' and index_id = ' + request.getValue('index_id'));
                    }

                    if(request.getValue('rights_enabled')){
                        _where.push(' and zn_create_user=' + (request.getSessionValueByKey("id")||'-1'));
                    }

                    this.collection('ZNPluginWorkflowInstance')
                        .paging({
                            fields: [
                                'id', 'code', 'state', 'zn_create_time', 'zn_modify_time', 'zn_note',
                                'zn_plugin_workflow_convert_index(index_id) as index_id_convert',
                                'zn_plugin_admin_convert_user(zn_create_user) as zn_create_user_convert',
                                'zn_plugin_workflow_convert_instance_current_state(id) as current_state'
                            ],
                            order: {
                                zn_create_time: 'desc'
                            },
                            where: _where
                        })
                        .then(function (data){
                            response.success(data);
                        }, function (err){
                            response.error(err);
                        });
                }
            },
            getInstance: {
                method: 'GET/POST',
                argv: {
                    instanceId: null
                },
                value: function (request, response, chain){
                    var _instanceId = request.getInt('instanceId'),
                        _sessionId = request.getSessionValueByKey("id"),
                        _return = {};
                    this.beginTransaction()
                        .query("select instance", function (){
                            var _sql = zn.sql.select({
                                table: 'zn_plugin_workflow_instance',
                                fields: [
                                    'zn_title', 'schema_uuid', 'next_schemas', 'zn_create_time', 'state', 'rights_logic', 'type', 'zn_rights_enabled', 'index_id',
                                    'zn_plugin_workflow_convert_schemas(next_schemas) as next_schemas_convert',
                                    'zn_plugin_admin_convert_user(zn_create_user) as zn_create_user_convert',
                                    'zn_plugin_admin_convert_user(zn_modify_user) as zn_modify_user_convert'
                                ],
                                where: { state: 1, parent_id: _instanceId }
                            });
                            _sql += zn.sql.select({
                                table: 'zn_plugin_workflow_instance',
                                fields: [
                                    'id', 'code', 'files', 'zn_create_time', 'zn_create_user', 'state', 'zn_note', 'zn_rights_enabled',
                                    'zn_plugin_workflow_convert_index(index_id) as index_id_convert',
                                    'zn_plugin_admin_convert_user(zn_create_user) as zn_create_user_convert',
                                    'zn_plugin_admin_convert_user(zn_modify_user) as zn_modify_user_convert',
                                    'zn_plugin_admin_user_exist(' + _sessionId + ', zn_rights_users, zn_rights_roles) as doRights',
                                    'zn_plugin_admin_user_exist(' + _sessionId + ', zn_rights_observe_users, zn_rights_observe_roles) as doObserveRights'
                                ],
                                where: { id: _instanceId }
                            });
                            _sql += zn.sql.select({
                                table: 'zn_plugin_workflow_instance',
                                fields: 'zn_title, zn_note, files, schema_form_data, state, type, zn_create_time, zn_create_user, zn_modify_time, zn_plugin_admin_convert_user(zn_modify_user) as zn_modify_user',
                                where: { parent_id: _instanceId }
                            });

                            return _sql;
                        }.bind(this))
                        .query("select form fields", function (sql, rows){
                            _return = {
                                current: rows[0][0],
                                base: rows[1][0],
                                data: rows[2]
                            };

                            if(_return.base.zn_create_user == _sessionId){
                                _return.base.doObserveRights = 1;
                                _return.base.isCreator = 1;
                            }

                            if(_return.current){
                                return zn.sql.select({
                                    table: 'zn_plugin_workflow_index_table',
                                    fields: ["id", "has_generated", "zn_title", "parent_id", "table_name"],
                                    where: { index_id: _return.current.index_id }
                                }) + zn.sql.select({
                                    table: 'zn_plugin_workflow_index_table_field',
                                    fields: ["id", "field_convert", "table_id", "title", "name", "type", "width", "data", "props"],
                                    where: { index_id: _return.current.index_id }
                                }) + zn.sql.select({
                                    table: 'zn_plugin_workflow_schema_field',
                                    fields: '*',
                                    where: [
                                        "schema_uuid='" +_return.current.schema_uuid + "'",
                                        ' and ' + zn.sql.rights()
                                    ]
                                });
                            }else {
                                return response.error("流程实例不存在, 请核实后再试！"), false;
                            }
                        })
                        .query('select form data', function (sql, rows){
                            if(rows[0].length){
                                var _tables = rows[0],
                        		 	_fields = rows[1],
                                    _primary = null,
                                    _table_data = [],
                        			_table_fields = {},
                                    _select_fields = {},
                                    _sqls = [];
                                _return.current.schema_fields = rows[2];
                        		_fields.forEach(function (field){
                        			if(!_table_fields[field.table_id]){
                        				_table_fields[field.table_id] = [];
                                        _select_fields[field.table_id] = [];
                        			}
                        			_table_fields[field.table_id].push(field);
                                    if(field.field_convert){
                                        _select_fields[field.table_id].push(field.field_convert + "(" + field.name + ") as " + field.name);
                                    }else {
                                        _select_fields[field.table_id].push(field.name);
                                    }
                        		});
                        		_tables.forEach(function (table){
                        			table.fields = _table_fields[table.id];
                                    if(table.has_generated){
                                        if(table.parent_id){
                                            _sqls.push(zn.sql.select({
                                                table: table.table_name,
                                                fields: _select_fields[table.id] || '*',
                                                where: {
                                                    zn_plugin_workflow_instance_id: _instanceId,
                                                    zn_plugin_workflow_index_id: _return.current.index_id
                                                }
                                            }));
                                            _table_data.push({
                                                zn_title: table.zn_title,
                                                fields: table.fields
                                            });
                            			}else {
                                            _sqls.unshift(zn.sql.select({
                                                table: table.table_name,
                                                fields: _select_fields[table.id] || '*',
                                                where: {
                                                    zn_plugin_workflow_instance_id: _instanceId,
                                                    zn_plugin_workflow_index_id: _return.current.index_id
                                                }
                                            }));
                                            _table_data.unshift({
                                                primary: true,
                                                zn_title: table.zn_title,
                                                fields: table.fields
                                            });
                            			}
                                    }
                        		});

                                _return.table_fields = _table_data;
                                return _sqls.join('');
                            }else {
                                return response.success(_return), false;
                            }
                        }.bind(this), function (err, rows){
                            if(err){
                                response.error(err);
                            }else {
                                _return.table_data = rows;
                                response.success(_return);
                            }
                        }).commit();
                }
            },
            start: {
                method: 'GET/POST',
                argv: {
                    instanceId: null
                },
                value: function (request, response, chain){
                    this.instanceService
                        .start(request.getValue('instanceId'))
                        .then(function (){
                            response.success('流程启动成功');
                        }, function (error){
                            response.success('流程启动失败：' + error.message);
                        });
                }
            },
            cancel: {
                method: 'GET/POST',
                argv: {
                    instanceId: null
                },
                value: function (request, response, chain){
                    this.instanceService
                        .cancel(request.getValue('instanceId'))
                        .then(function (){
                            response.success('流程取消成功');
                        }, function (error){
                            response.success('流程取消启动失败：' + error.message);
                        });
                }
            },
            next: {
                method: 'GET/POST',
                argv: {
                    instanceId: null,
                    nextSchemaUUID: null,
                    options: null
                },
                value: function (request, response, chain){
                    console.log(request.getJSON('options'));
                    this.instanceService
                        .next(request.getValue('instanceId'), request.getValue('nextSchemaUUID'), request.getJSON('options'))
                        .then(function (data){
                            response.success(data);
                        }, function (error){
                            response.error('调用错误：' + error.message);
                        });
                }
            }
        }
    });

});
